#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>

// Error codes.
#define RDS_ERR_OK          0
#define RDS_ERR_CANTOPEN    1
#define RDS_ERR_CANTMALLOC  2
#define RDS_ERR_CMDSYNTAX   3
#define RDS_ERR_CANTWRITE   4
#define RDS_ERR_NOTOPEN     5
#define RDS_ERR_CANTREAD    6
#define RDS_ERR_QUERYSYNTAX 7
#define RDS_NUM_ERR         8 // Count

// Error return values.
#define RDS_IERRVAL (-99999999)
#define RDS_FERRVAL (-9999999.9f)

class Rigol_DS1000
{
public:
  Rigol_DS1000( int ifnum=0 );
  virtual ~Rigol_DS1000();

  // Low level, general purpose, communication.
  bool command( const char* cmd );
  char* string_query( const char* text );
  int int_query( const char* text );
  float float_query( const char* text );

  // Acquire and return waveform data.
  bool acquire_channels( int* chan1, int* chan2, int* npts, bool stop=true, bool golocal=true );
  int get_channel( int chan, float** vert, float* deltat, float* hoffset, float* voffset );

  // Get last error.
  const char* error_string( void );

private:
  int read_into_wavebuffer( void );

public:
  int lasterr;               // Last error code.
  bool opened;               // Device /dev/usbtmcN opened OK.
  int fd;                    // File descriptor for device i/o.
  int maxsamples;            // Maximum possible waveform samples.
  unsigned char* wavebuffer; // Buffer to store raw waveform data.
  char buffer[2048];         // Multi-purpose buffer.
  int count;                 // I/o transfer count.
  bool chan1_p;              // Channel 1 available flag.
  float* ch1_wave;           // Ch1 waveform as volts.
  int ch1_nsamples;          // Number of samples in Ch1 waveform.
  float ch1_vscale;          // Vertical scale: Ch1 volts/div.
  float ch1_voffset;         // Vertical offset: Ch1 offset on display in volts.
  bool chan2_p;              // Channel 2 available flag.
  float* ch2_wave;           // Ch2 waveform as volts.
  int ch2_nsamples;          // Number of samples in Ch2 waveform.
  float ch2_vscale;          // Vertical scale: Ch2 volts/div.
  float ch2_voffset;         // Vertical offset: Ch2 offset on display in volts.
  float hscale;              // Timebase horizontal scale: secs/div.
  float hoffset;             // Timebase horizontal offset: secs.
  float ch1_deltat;          // Ch1 secs between samples.
  float ch2_deltat;          // Ch2 secs between samples.
};


Rigol_DS1000::Rigol_DS1000( int ifnum )
//-------------------------------------
// Open a Rigol DS1000 series oscilloscope in /dev/usbtmc<ifnum>
{
  lasterr = RDS_ERR_OK;
  opened = false;
  fd = -1;
  maxsamples = 2 * 1024 * 1024;
  wavebuffer = NULL;
  ch1_wave = NULL;
  ch2_wave = NULL;
  ch1_nsamples = 0;
  ch2_nsamples = 0;
  chan1_p = false;
  chan2_p = false;
  ch1_vscale = 0.0f;
  ch1_voffset = 0.0f;
  ch2_vscale = 0.0f;
  ch2_voffset = 0.0f;
  hscale = 0.0f;
  hoffset = 0.0f;
  ch1_deltat = 0.0f;
  ch2_deltat = 0.0f;
  memset( buffer, 0, 2048 );
  sprintf( buffer, "/dev/usbtmc%d", ifnum );
  fd = open( buffer, O_RDWR );
  if( fd < 0 ){
    lasterr = RDS_ERR_CANTOPEN;
    return;
  }
  wavebuffer = (unsigned char*)malloc( maxsamples * sizeof(unsigned char) );
  ch1_wave = (float*)malloc( maxsamples * sizeof(float) );
  ch2_wave = (float*)malloc( maxsamples * sizeof(float) );
  opened = ( wavebuffer != NULL && ch1_wave != NULL && ch2_wave != NULL );
  if( ! opened )
    lasterr = RDS_ERR_CANTMALLOC;
}

Rigol_DS1000::~Rigol_DS1000()
//---------------------------
// Clean up.
{
  if( wavebuffer != NULL )
    free( wavebuffer );
  if( ch1_wave != NULL )
    free( ch1_wave );
  if( ch2_wave != NULL )
    free( ch2_wave );
  if( fd >= 0 )
    close( fd );
}

const char* Rigol_DS1000::error_string( void )
//--------------------------------------
// Return a description of the last error and reset last error.
// Return NULL if no error.
{
  static const char* errstrings[] = {
    NULL,                                          // RDS_ERR_OK          0
    "Cannot open device /dev/usbtmc...",           // RDS_ERR_CANTOPEN    1
    "Cannot allocate memory.",                     // RDS_ERR_CANTMALLOC  2
    "Invalid command syntax.",                     // RDS_ERR_CMDSYNTAX   3
    "Cannot write to /dev/usbtmc...",              // RDS_ERR_CANTWRITE   4
    "/dev/usbtmc... was not opened successfully.", // RDS_ERR_NOTOPEN     5
    "Cannot read from /dev/usbtmc...",             // RDS_ERR_CANTREAD    6
    "Invalid query syntax."                        // RDS_ERR_QUERYSYNTAX 7
  };
  if( lasterr >=0 && lasterr < RDS_NUM_ERR ){
    int slasterr = lasterr;
    lasterr = RDS_ERR_OK;
    return( errstrings[slasterr] );
  }
  else
    return( "Unknown error." );
}

bool Rigol_DS1000::command( const char* cmd )
//-------------------------------------------
// Send a command string to the scope. Check string begins with :.
// Return false on error.
{
  if( opened ){
    if( cmd[0] != ':' && cmd[0] != '*' ){
      lasterr = RDS_ERR_CMDSYNTAX;
      return( false );
    }
    strncpy( buffer, cmd, 2047 );
    if( write( fd, buffer, strlen(buffer) ) < 0 ){
      lasterr = RDS_ERR_CANTWRITE;
      return( false );
    }
    return( true );
  }
  else{
    lasterr = RDS_ERR_NOTOPEN;
    return( false );
  }
}

char* Rigol_DS1000::string_query( const char* text )
//--------------------------------------------------
// Send text to the scope. text must contain a ?.
// Return NULL on error.
{
  if( opened ){
    if( strrchr( text, '?' ) != NULL ){
      if( ! command( text ) ){
	return( NULL );
      }
      count = read( fd, buffer, 2047 );
      if( count < 0 ){
	lasterr = RDS_ERR_CANTREAD;
	return( NULL );
      }
      else{
	buffer[count] = 0;
	printf( "count=%d\n%s\n", count, buffer );
	return( buffer );
      }
    }
    else{
      lasterr = RDS_ERR_QUERYSYNTAX;
      return( NULL );
    }
  }
  else{
    lasterr = RDS_ERR_NOTOPEN;
    return( NULL );
  }
}

int Rigol_DS1000::int_query( const char* text )
//---------------------------------------------
// Return the result of a query (text) as an integer.
// Return RDS_IERRVAL on error.
{
  char* cresult = string_query( text );
  if( cresult == NULL )
    return( RDS_IERRVAL );
  return( atoi( cresult ) );
}

float Rigol_DS1000::float_query( const char* text )
//-------------------------------------------------
// Return the result of a query (text) as a float.
// Return RDS_FERRVAL on error.
{
  char* cresult = string_query( text );
  if( cresult == NULL )
    return( RDS_FERRVAL );
  return( atof( cresult ) );
}


int Rigol_DS1000::read_into_wavebuffer( void )
//--------------------------------------------
// Read waveform data from the oscilloscope as bytes in small chunks.
// Trying to read the full number of samples in one lump will crash the scope
// probably due to internal buffer overflow somewhere. 1K works fine, though, and
// 2K would probably be OK too.
// The byte values are display coordinates, inverted in Y, with screen center at 125.
// This is not documented AFAIK, but it seems to be the case. The only hint in the manual
// is that the dynamic range is 5 divisions.
// Return the number of samples read.
{
  const int chunk = 1024;
  unsigned char lbuffer[chunk];
  char csamples[10];

  // Get the number of samples available.
  int count = read( fd, lbuffer, 10 );
  if( count < 0 ){
    lasterr = RDS_ERR_CANTREAD;
    return( 0 );
  }
  strncpy( csamples, (const char *)&lbuffer[2], 8 );
  csamples[9] = 0;
  int nsamples = atoi( csamples );
  printf( "readwave nsamples = %d\n", nsamples );

  // Read the data in chunk sized chunks. Accumulate the full record
  // in wavebuffer.
  int nleft = nsamples;
  int insert = 0;
  for( ; nleft>0; nleft-=chunk,insert+=chunk ){
    int nread = (nleft > chunk) ? chunk : nleft ;
    count = read( fd, lbuffer, nread );
    if( count < 0 ){
      lasterr = RDS_ERR_CANTREAD;
      return( 0 );
    }
    memcpy( &wavebuffer[insert], lbuffer, nread );
  }

  //for( int i=0; i<nsamples && i < 2048; i++ ){
  //  printf( "%d,", (int)buffer[i]-128 );
  //}
  return( nsamples );
}

bool Rigol_DS1000::acquire_channels( int* chan1, int* chan2, int* npts, bool stop, bool golocal )
//-----------------------------------------------------------------------------------------------
// Acquire waveform data from the oscilloscope.
// Return which channels are available and how many points will be returned from them.
// The number of samples for each channel are returned in chan1, chan2 (0 if not available).
// The maximum of the two channels sample count is returned in npts.
// If stop, stop before acquiring, then restart.
// If golocal, leave remote mode after acquisition.
{
  *chan1 = 0;
  *chan2 = 0;
  *npts = 0;
  if( ! opened )
    return( false );

  // See which channels are on.
  char* chanq = string_query( ":CHAN1:DISP?" );
  if( chanq == NULL )
    return( false );
  chan1_p = ( strcmp( chanq, "1" ) == 0 );
  chanq = string_query( ":CHAN2:DISP?" );
  if( chanq == NULL )
    return( false );
  chan2_p = ( strcmp( chanq, "1" ) == 0 );

  // Stop acquisition.
  if( stop ){
    if( ! command( ":STOP" ) )
      return( false );
  }
  usleep( 20000 );

  // Get the timebase information.
  hscale = float_query( ":TIM:SCAL?" );
  hoffset = float_query( ":TIM:OFFS?" );
  printf( "timebase: hscale=%f, hoffset=%f\n", hscale, hoffset );

  // Read channel 1 if it is available.
  if( chan1_p ){
    printf( "chan1 available\n" );
    ch1_vscale = float_query( ":CHAN1:SCAL?" );
    ch1_voffset = float_query( ":CHAN1:OFFS?" );
    float samprate = float_query(":ACQ:SAMP? CHAN1" );
    ch1_deltat = 1.0f / samprate;
    printf( "ch1: vscale=%f, voffset=%f, deltat=%f\n", ch1_vscale, ch1_voffset, ch1_deltat );
    command( ":WAV:DATA? CHAN1" );
    ch1_nsamples = read_into_wavebuffer();
    if( ch1_nsamples > 0 ){
      // Convert display coordinates to volts. This seems to work! But the format is not
      // actually documented anywhere AFAIK.
      for( int i=0; i<ch1_nsamples; i++ ){
	ch1_wave[i] = ( (float)(125 - wavebuffer[i]) / 25.0f * ch1_vscale ) - ch1_voffset;
      }
    }
    else{
      return( false );
    }
    *chan1 = ch1_nsamples;
  }

  // Read channel 2 if it is available.
  if( chan2_p ){
    printf( "chan2 available\n" );
    ch2_vscale = float_query( ":CHAN2:SCAL?" );
    ch2_voffset = float_query( ":CHAN2:OFFS?" );
    float samprate = float_query(":ACQ:SAMP? CHAN2" );
    ch2_deltat = 1.0f / samprate;
    printf( "ch2: vscale=%f, voffset=%f, deltat=%f\n", ch1_vscale, ch1_voffset, ch1_deltat );
    ch2_nsamples = read_into_wavebuffer();
    if( ch2_nsamples > 0 ){
      for( int i=0; i<ch2_nsamples; i++ ){
	ch2_wave[i] = ( (float)(125 - wavebuffer[i]) / 25.0f * ch2_vscale ) - ch2_voffset;
      }
    }
    else{
      return( false );
    }
    *chan2 = ch2_nsamples;
  }

  // Restart acquisition.
  if( stop ){
    if( ! command( ":RUN" ) )
      return( false );
  }

  // Enable local keys (end remote).
  if( golocal ){
    if( ! command( ":KEY:FORC" ) )
      return( false );
  }

  // Return the maximum number of samples.
  *npts = (ch1_nsamples > ch2_nsamples) ? ch1_nsamples : ch2_nsamples ;
  return( true );
}

  int Rigol_DS1000::get_channel( int chan, float** vert, float* deltat, float* uhoffset, float* voffset )
//-------------------------------------------------------------------------------------------------------
// Return the specified channel data. The waveform is returned thru vert with its sampling rate,
// horizontal and vertical offsets in deltat, uhoffset and voffset. The number of samples is the
// return value (0 if the channel is not available).
{
  if( ! opened )
    return( 0 );
  if( chan == 1 ){
    if( ! chan1_p )
      return( 0 );
    *vert = ch1_wave;
    *deltat = ch1_deltat;
    *uhoffset = hoffset;
    *voffset = ch1_voffset;
    return( ch1_nsamples );
  }
  else if( chan == 2 ){
    if( ! chan2_p )
      return( 0 );
    *vert = ch2_wave;
    *deltat = ch2_deltat;
    *uhoffset = hoffset;
    *voffset = ch2_voffset;
    return( ch2_nsamples );
  }
  else
    return( 0 );
}

void save_data( int n, float* data, float deltat, const char* fnamestem, int chan )
//---------------------------------------------------------------------------------
// Save the n points of data in file fnamestem_ch<chan>.txt as two columns: time value
// This will allow it to be drawn for debug purposes.
{
  char fname[512];
  sprintf( fname, "%s_ch%d.txt", fnamestem, chan );

  FILE* file = fopen( fname, "w+" );
  if( file == NULL ){
    fprintf( stderr, "Cannot open: %s\n", fname );
    return;
  }
  for( int i=0; i<n; i++ ){
    fprintf( file, "%f %f\n", i*deltat, data[i] );
  }
  fclose( file );
}

int split_string( char* string, char** parts, int maxparts )
//-----------------------------------------------------------
// Split string in to parts delimited by ;. Return up to maxparts pointers
// to the parts in parts[]. Return the number of parts.
// String is modified here.
{
  if( maxparts == 0 )
    return( 0 );
  int len = (int)strlen( string );
  int nparts = 0;
  parts[nparts] = string;
  nparts++;
  for( int i=0; i<len; i++ ){
    if( string[i] == ';' ){
      string[i] = 0;
      i++;
      if( string[i] == 0 || nparts == maxparts )
	return( nparts );
      parts[nparts] = &string[i];
      nparts++;
    }
  }
  return( nparts );
}

int main( int argc, char** argv )
{
  char outfilestem[512];

  if( argc < 2 ){
    fprintf( stderr, "Usage: gencmd GET [filenamestem] | commands\n"
                     "GET reads waveforms back from the scope to file.\n"
	     "commands is a set of valid commands separated by ;\n" );
    return( 1 );
  }

  if( argc == 3 )
    sprintf( outfilestem, "%s", argv[2] );
  else
    sprintf( outfilestem, "/tmp/rigol" );

  // Open communications with the oscilloscope.
  Rigol_DS1000* scope = new Rigol_DS1000( 0 );
  if( scope != NULL ){
    const char* errstr = scope->error_string();
    if( errstr != NULL ){
      fprintf( stderr, "Error: %s\n", errstr );
      return( 1 );
    }

    // If command is GET, acquire and read waveforms.
    if( strstr( argv[1], "GET" ) != NULL ){
      int chan1pts, chan2pts, maxpts;
      if( scope->acquire_channels( &chan1pts, &chan2pts, &maxpts ) ){
	float* vert, deltat, hoffset, voffset;
	printf( "Channel points: CH1:%d, CH2:%d, MAX:%d\n", chan1pts, chan2pts, maxpts );
	if( chan1pts > 0 ){
	  scope->get_channel( 1, &vert, &deltat, &hoffset, &voffset );
	  save_data( chan1pts, vert, deltat, outfilestem, 1 );
	}
	if( chan2pts > 0 ){
	  scope->get_channel( 2, &vert, &deltat, &hoffset, &voffset );
	  save_data( chan1pts, vert, deltat, outfilestem, 2 );
	}
      }
      else{
	errstr = scope->error_string();
	if( errstr != NULL )fprintf( stderr, "Error: %s\n", errstr );
	return( 1 );
      }
    }

    else{
      // Split the input commands up. Allow up to 20.
      char* cmds[20];
      char incmd[1024];
      memset( incmd, 0, 1024 );
      strncpy( incmd, argv[1], 1023 );
      int nparts = split_string( incmd, cmds, 20 );
      for( int i = 0; i<nparts; i++ ){
	printf( "%d: %s\n", i, cmds[i] );

	// If command is a query, do a string query.
	if( strrchr( cmds[i], '?' ) != NULL ){
	  char* reply = scope->string_query( cmds[i] );
	  if( reply == NULL ){
	    errstr = scope->error_string();
	    if( errstr != NULL )fprintf( stderr, "Error: %s\n", errstr );
	    return( 1 );
	  }
	  else{
	    printf( "Reply=\"%s\"\n", reply );
	    fflush( stdout );
	  }
	}

	// Otherwise, send a command.
	else{
	  if( ! scope->command( cmds[i] ) ){
	    errstr = scope->error_string();
	    if( errstr != NULL )fprintf( stderr, "Error: %s\n", errstr );
	    return( 1 );
	  }
	}
      }
    }
  } // Over nparts cmds.

  return( 0 );
}
