// rigol_DS1000.cpp - Support for controlling Rigol Technologies DS1000 series oscilloscopes.
// ==========================================================================================
//
// This uses the /dev/usbtmc interface. This is built in to recent Ubuntu kernels.
// It is certainly there in 12.04 LTS anyway.
//

#include "rigol_ds1000.h"

Rigol_DS1000::Rigol_DS1000( int ifnum )
//-------------------------------------
// Open a Rigol DS1000 series oscilloscope in /dev/usbtmc<ifnum>
{
  lasterr = RDS_ERR_OK;
  opened = false;
  fd = -1;
  maxsamples = 2 * 1024 * 1024;
  wavebuffer = NULL;
  ch1_wave = NULL;
  ch2_wave = NULL;
  ch1_nsamples = 0;
  ch2_nsamples = 0;
  chan1_p = false;
  chan2_p = false;
  ch1_vscale = 0.0f;
  ch1_voffset = 0.0f;
  ch2_vscale = 0.0f;
  ch2_voffset = 0.0f;
  hscale = 0.0f;
  hoffset = 0.0f;
  ch1_deltat = 0.0f;
  ch2_deltat = 0.0f;
  memset(message, 0, 1024 );
  memset( buffer, 0, 2048 );
  //debugon( true );
  sprintf( buffer, "/dev/usbtmc%d", ifnum );
  fd = open( buffer, O_RDWR );
  if( fd < 0 ){
    lasterr = RDS_ERR_CANTOPEN;
    return;
  }
  wavebuffer = (unsigned char*)malloc( maxsamples * sizeof(unsigned char) );
  ch1_wave = (float*)malloc( maxsamples * sizeof(float) );
  ch2_wave = (float*)malloc( maxsamples * sizeof(float) );
  opened = ( wavebuffer != NULL && ch1_wave != NULL && ch2_wave != NULL );
  if( ! opened )
    lasterr = RDS_ERR_CANTMALLOC;
}

Rigol_DS1000::~Rigol_DS1000()
//---------------------------
// Clean up.
{
  if( wavebuffer != NULL )
    free( wavebuffer );
  if( ch1_wave != NULL )
    free( ch1_wave );
  if( ch2_wave != NULL )
    free( ch2_wave );
  if( fd >= 0 )
    close( fd );
}

void Rigol_DS1000::notify( const char* fmt, ... )
//-----------------------------------------------
// Output a message via notify_out().
{
  va_list args;

  va_start( args, fmt );
  vsprintf( message, fmt, args );
  va_end( args );
  notify_out();
}

void Rigol_DS1000::notify_out( void )
//-----------------------------------
// Output a message string.
{
  fprintf( stderr, "%s", message );
}

const char* Rigol_DS1000::error_string( void )
//--------------------------------------------
// Return a description of the last error and reset last error.
// Return NULL if no error.
{
  static const char* errstrings[] = {
    NULL,                                          // RDS_ERR_OK          0
    "Cannot open device /dev/usbtmc...",           // RDS_ERR_CANTOPEN    1
    "Cannot allocate memory.",                     // RDS_ERR_CANTMALLOC  2
    "Invalid command syntax.",                     // RDS_ERR_CMDSYNTAX   3
    "Cannot write to /dev/usbtmc...",              // RDS_ERR_CANTWRITE   4
    "/dev/usbtmc... was not opened successfully.", // RDS_ERR_NOTOPEN     5
    "Cannot read from /dev/usbtmc...",             // RDS_ERR_CANTREAD    6
    "Invalid query syntax."                        // RDS_ERR_QUERYSYNTAX 7
  };
  if( lasterr >=0 && lasterr < RDS_NUM_ERR ){
    int slasterr = lasterr;
    lasterr = RDS_ERR_OK;
    return( errstrings[slasterr] );
  }
  else
    return( "Unknown error." );
}

bool Rigol_DS1000::command( const char* cmd )
//-------------------------------------------
// Send a command string to the scope. Check string begins with :.
// Return false on error.
{
  if( opened ){
    if( cmd[0] != ':' && cmd[0] != '*' ){
      lasterr = RDS_ERR_CMDSYNTAX;
      return( false );
    }
    strncpy( buffer, cmd, 2047 );
    if( write( fd, buffer, strlen(buffer) ) < 0 ){
      lasterr = RDS_ERR_CANTWRITE;
      return( false );
    }
    return( true );
  }
  else{
    lasterr = RDS_ERR_NOTOPEN;
    return( false );
  }
}

int Rigol_DS1000::split_string( char* string, char** parts, int maxparts )
//------------------------------------------------------------------------
// Split string in to parts delimited by ;. Return up to maxparts pointers
// to the parts in parts[]. Return the number of parts.
// String is modified here.
{
  if( maxparts == 0 )
    return( 0 );
  int len = (int)strlen( string );
  int nparts = 0;
  parts[nparts] = string;
  nparts++;
  for( int i=0; i<len; i++ ){
    if( string[i] == ';' ){
      string[i] = 0;
      i++;
      if( string[i] == 0 || nparts == maxparts )
	return( nparts );
      parts[nparts] = &string[i];
      nparts++;
    }
  }
  return( nparts );
}

bool Rigol_DS1000::multi_command( const char* manycmds )
//------------------------------------------------------
// Execute a sequence of commands separated by ;'s.
// Queries are allowed and the results output as diagnostics,
// but, generally, this will not be useful.
{
  const char* errstr;

  // Split the input commands up. Allow up to 80.
  char* cmds[80];
  char incmd[1024];
  memset( incmd, 0, 1024 );
  strncpy( incmd, manycmds, 1023 );
  int nparts = split_string( incmd, cmds, 80 );
  for( int i = 0; i<nparts; i++ ){
    if( debugmode )
      notify( "%d: %s\n", i, cmds[i] );

    // If command is a query, do a string query.
    if( strrchr( cmds[i], '?' ) != NULL ){
      char* reply = string_query( cmds[i] );
      if( reply == NULL ){
	errstr = error_string();
	if( errstr != NULL && debugmode )
	  notify( "Error: %s\n", errstr );
	return( false );
      }
      else{
	notify( "Reply=\"%s\"\n", reply );
      }
    }

    // Otherwise, send a command.
    else{
      if( ! command( cmds[i] ) ){
	errstr = error_string();
	if( errstr != NULL && debugmode )
	  notify( "Error: %s\n", errstr );
	return( false );
      }
    }
  }

  return( true );
}

char* Rigol_DS1000::string_query( const char* text )
//--------------------------------------------------
// Send text to the scope. text must contain a ?.
// Return NULL on error.
{
  if( opened ){
    if( strrchr( text, '?' ) != NULL ){
      if( ! command( text ) ){
	return( NULL );
      }
      count = read( fd, buffer, 2047 );
      if( count < 0 ){
	lasterr = RDS_ERR_CANTREAD;
	return( NULL );
      }
      else{
	buffer[count] = 0;
	if( debugmode )
	  notify( "count=%d\n%s\n", count, buffer );
	return( buffer );
      }
    }
    else{
      lasterr = RDS_ERR_QUERYSYNTAX;
      return( NULL );
    }
  }
  else{
    lasterr = RDS_ERR_NOTOPEN;
    return( NULL );
  }
}

int Rigol_DS1000::int_query( const char* text )
//---------------------------------------------
// Return the result of a query (text) as an integer.
// Return RDS_IERRVAL on error.
{
  char* cresult = string_query( text );
  if( cresult == NULL )
    return( RDS_IERRVAL );
  return( atoi( cresult ) );
}

float Rigol_DS1000::float_query( const char* text )
//-------------------------------------------------
// Return the result of a query (text) as a float.
// Return RDS_FERRVAL on error.
{
  char* cresult = string_query( text );
  if( cresult == NULL )
    return( RDS_FERRVAL );
  return( atof( cresult ) );
}

int Rigol_DS1000::read_into_wavebuffer( void )
//--------------------------------------------
// Read waveform data from the oscilloscope as bytes in small chunks.
// Trying to read the full number of samples in one lump will crash the scope
// probably due to internal buffer overflow somewhere. 1K works fine, though, and
// 2K would probably be OK too.
// The byte values are display coordinates, inverted in Y, with screen center at 125.
// This is not documented AFAIK, but it seems to be the case. The only hint in the manual
// is that the dynamic range is 5 divisions.
// Return the number of samples read.
{
  const int chunk = 1024;
  unsigned char lbuffer[chunk];
  char csamples[10];

  // Get the number of samples available.
  int count = read( fd, lbuffer, 10 );
  if( count < 0 ){
    lasterr = RDS_ERR_CANTREAD;
    return( 0 );
  }
  strncpy( csamples, (const char *)&lbuffer[2], 8 );
  csamples[9] = 0;
  int nsamples = atoi( csamples );
  if( debugmode )
    notify( "readwave nsamples = %d\n", nsamples );

  // Read the data in chunk sized chunks. Accumulate the full record
  // in wavebuffer.
  int nleft = nsamples;
  int insert = 0;
  for( ; nleft>0; nleft-=chunk,insert+=chunk ){
    int nread = (nleft > chunk) ? chunk : nleft ;
    count = read( fd, lbuffer, nread );
    if( count < 0 ){
      lasterr = RDS_ERR_CANTREAD;
      return( 0 );
    }
    memcpy( &wavebuffer[insert], lbuffer, nread );
  }

  return( nsamples );
}

bool Rigol_DS1000::acquire_channels( int* chan1, int* chan2, int* npts, bool stop, bool golocal )
//-----------------------------------------------------------------------------------------------
// Acquire waveform data from the oscilloscope.
// Return which channels are available and how many points will be returned from them.
// The number of samples for each channel are returned in chan1, chan2 (0 if not available).
// The maximum of the two channels sample count is returned in npts.
// If stop, stop before acquiring, then restart.
// If golocal, leave remote mode after acquisition.
{
  *chan1 = 0;
  *chan2 = 0;
  *npts = 0;
  if( ! opened )
    return( false );

  // See which channels are on.
  char* chanq = string_query( ":CHAN1:DISP?" );
  if( chanq == NULL )
    return( false );
  chan1_p = ( strcmp( chanq, "1" ) == 0 );
  chanq = string_query( ":CHAN2:DISP?" );
  if( chanq == NULL )
    return( false );
  chan2_p = ( strcmp( chanq, "1" ) == 0 );

  // Stop acquisition.
  if( stop ){
    if( ! command( ":STOP" ) )
      return( false );
  }
  usleep( 100000 );

  // Get the timebase information.
  hscale = float_query( ":TIM:SCAL?" );
  hoffset = float_query( ":TIM:OFFS?" );
  if( debugmode )
    notify( "timebase: hscale=%f, hoffset=%f\n", hscale, hoffset );

  // Read channel 1 if it is available.
  if( chan1_p ){
    if( debugmode )
      notify( "chan1 available\n" );
    ch1_vscale = float_query( ":CHAN1:SCAL?" );
    ch1_voffset = float_query( ":CHAN1:OFFS?" );
    float samprate = float_query(":ACQ:SAMP? CHAN1" );
    ch1_deltat = 1.0f / samprate;
    if( debugmode )
      notify( "ch1: vscale=%f, voffset=%f, deltat=%f\n", ch1_vscale, ch1_voffset, ch1_deltat );
    command( ":WAV:DATA? CHAN1" );
    ch1_nsamples = read_into_wavebuffer();
    if( ch1_nsamples > 0 ){
      // Convert display coordinates to volts. This seems to work! But the format is not
      // actually documented anywhere AFAIK.
      for( int i=0; i<ch1_nsamples; i++ ){
	ch1_wave[i] = ( (float)(125 - wavebuffer[i]) / 25.0f * ch1_vscale ) - ch1_voffset;
      }
    }
    else{
      return( false );
    }
    *chan1 = ch1_nsamples;
  }

  // Read channel 2 if it is available.
  if( chan2_p ){
    if( debugmode )
      notify( "chan2 available\n" );
    ch2_vscale = float_query( ":CHAN2:SCAL?" );
    ch2_voffset = float_query( ":CHAN2:OFFS?" );
    float samprate = float_query(":ACQ:SAMP? CHAN2" );
    ch2_deltat = 1.0f / samprate;
    if( debugmode )
      notify( "ch2: vscale=%f, voffset=%f, deltat=%f\n", ch1_vscale, ch1_voffset, ch1_deltat );
    command( ":WAV:DATA? CHAN2" );
    ch2_nsamples = read_into_wavebuffer();
    if( ch2_nsamples > 0 ){
      for( int i=0; i<ch2_nsamples; i++ ){
	ch2_wave[i] = ( (float)(125 - wavebuffer[i]) / 25.0f * ch2_vscale ) - ch2_voffset;
      }
    }
    else{
      return( false );
    }
    *chan2 = ch2_nsamples;
  }

  // Restart acquisition.
  if( stop ){
    if( ! command( ":RUN" ) )
      return( false );
  }

  // Enable local keys (end remote).
  if( golocal ){
    if( ! command( ":KEY:FORC" ) )
      return( false );
  }

  // Return the maximum number of samples.
  *npts = (ch1_nsamples > ch2_nsamples) ? ch1_nsamples : ch2_nsamples ;
  return( true );
}

int Rigol_DS1000::get_channel( int chan, float** vert, float* deltat, float* uhoffset, float* voffset )
//-----------------------------------------------------------------------------------------------------
// Return the specified channel data. The waveform is returned thru vert with its sampling rate,
// horizontal and vertical offsets in deltat, uhoffset and voffset. The number of samples is the
// return value (0 if the channel is not available).
{
  if( ! opened )
    return( 0 );
  if( chan == 1 ){
    if( ! chan1_p )
      return( 0 );
    *vert = ch1_wave;
    *deltat = ch1_deltat;
    *uhoffset = hoffset;
    *voffset = ch1_voffset;
    return( ch1_nsamples );
  }
  else if( chan == 2 ){
    if( ! chan2_p )
      return( 0 );
    *vert = ch2_wave;
    *deltat = ch2_deltat;
    *uhoffset = hoffset;
    *voffset = ch2_voffset;
    return( ch2_nsamples );
  }
  else
    return( 0 );
}


// C bindings for Python Ctypes.
extern "C" {
  Rigol_DS1000* Rigol_DS1000_new( int ifnum )
  {return new Rigol_DS1000( ifnum );}
  int Rigol_DS1000_error( Rigol_DS1000* s )
  {return((s->error_string())==NULL?0:1);}
  char* Rigol_DS1000_error_string( Rigol_DS1000* s )
  {return((char*)s->error_string());}
  int Rigol_DS1000_command( Rigol_DS1000* s, char* cmd )
  {return(s->command( (const char*)cmd )?1:0);}
  int Rigol_DS1000_multi_command( Rigol_DS1000* s, char* cmds )
  {return(s->multi_command( (const char*)cmds )?1:0);}
  char* Rigol_DS1000_string_query( Rigol_DS1000* s, char* text )
  {return(s->string_query( (const char*)text ));}
  int Rigol_DS1000_acquire_channels( Rigol_DS1000* s, int* chan1, int* chan2, int* npts, int stop, int golocal )
  {return(s->acquire_channels(chan1, chan2, npts, (stop!=0), (golocal!=0) ));}
  int Rigol_DS1000_get_channel( Rigol_DS1000* s, int chan, float* vert, float* deltat, float* uhoffset, float* voffset )
  {
    float* vin; 
    int n = s->get_channel(chan, &vin, deltat, uhoffset, voffset );
    for( int i=0; i<n; i++ )
      vert[i] = vin[i];
    return( n );
  }
}
