#ifndef __RIGOL_DS1000_H__
#define __RIGOL_DS1000_H__

//
// rigol_DS1000.h - Support for controlling Rigol Technologies DS1000 series oscilloscopes.
// ========================================================================================
//
// This uses the /dev/usbtmc interface. This is built in to recent Ubuntu kernels.
// It is certainly there in 12.04 LTS anyway.
//

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>
#include <stdarg.h>

// Error codes.
#define RDS_ERR_OK          0
#define RDS_ERR_CANTOPEN    1
#define RDS_ERR_CANTMALLOC  2
#define RDS_ERR_CMDSYNTAX   3
#define RDS_ERR_CANTWRITE   4
#define RDS_ERR_NOTOPEN     5
#define RDS_ERR_CANTREAD    6
#define RDS_ERR_QUERYSYNTAX 7
#define RDS_NUM_ERR         8 // Count

// Error return values.
#define RDS_IERRVAL (-99999999)
#define RDS_FERRVAL (-9999999.9f)

class Rigol_DS1000
{
public:
  Rigol_DS1000( int ifnum=0 );
  virtual ~Rigol_DS1000();

  // Diagnostic message i/o.
  void debugon( bool yes ) {debugmode = yes;}
  void notify( const char* fmt, ... );
  virtual void notify_out( void );

  // Low level, general purpose, communication.
  bool command( const char* cmd );
  bool multi_command( const char* manycmds );
  char* string_query( const char* text );
  int int_query( const char* text );
  float float_query( const char* text );

  // Acquire and return waveform data.
  bool acquire_channels( int* chan1, int* chan2, int* npts, bool stop=true, bool golocal=true );
  int get_channel( int chan, float** vert, float* deltat, float* hoffset, float* voffset );

  // Get last error.
  const char* error_string( void );

  // Sundries.
  int split_string( char* string, char** parts, int maxparts );

private:
  int read_into_wavebuffer( void );

public:
  int lasterr;               // Last error code.
  bool opened;               // Device /dev/usbtmcN opened OK.
  int fd;                    // File descriptor for device i/o.
  int maxsamples;            // Maximum possible waveform samples.
  unsigned char* wavebuffer; // Buffer to store raw waveform data.
  char buffer[2048];         // Multi-purpose buffer.
  int count;                 // I/o transfer count.
  bool chan1_p;              // Channel 1 available flag.
  float* ch1_wave;           // Ch1 waveform as volts.
  int ch1_nsamples;          // Number of samples in Ch1 waveform.
  float ch1_vscale;          // Vertical scale: Ch1 volts/div.
  float ch1_voffset;         // Vertical offset: Ch1 offset on display in volts.
  bool chan2_p;              // Channel 2 available flag.
  float* ch2_wave;           // Ch2 waveform as volts.
  int ch2_nsamples;          // Number of samples in Ch2 waveform.
  float ch2_vscale;          // Vertical scale: Ch2 volts/div.
  float ch2_voffset;         // Vertical offset: Ch2 offset on display in volts.
  float hscale;              // Timebase horizontal scale: secs/div.
  float hoffset;             // Timebase horizontal offset: secs.
  float ch1_deltat;          // Ch1 secs between samples.
  float ch2_deltat;          // Ch2 secs between samples.
  char message[1024];        // Diagnostic message buffer.
  bool debugmode;            // Output debug diagnostics.
};

#endif //__RIGOL_DS1000_H__
