//
// rigolcmd.cpp - Simple test program for the rigol_ds1000 control code.
// =====================================================================
//

#include "rigol_ds1000.h"

void save_data( int n, float* data, float deltat, const char* fnamestem, int chan )
//---------------------------------------------------------------------------------
// Save the n points of data in file fnamestem_ch<chan>.txt as two columns: time value
// This will allow it to be drawn for debug purposes.
{
  char fname[512];
  sprintf( fname, "%s_ch%d.txt", fnamestem, chan );

  FILE* file = fopen( fname, "w+" );
  if( file == NULL ){
    fprintf( stderr, "Cannot open: %s\n", fname );
    return;
  }
  for( int i=0; i<n; i++ ){
    fprintf( file, "%f %f\n", i*deltat, data[i] );
  }
  fclose( file );
}

int main( int argc, char** argv )
{
  char outfilestem[512];

  if( argc < 2 ){
    fprintf( stderr, "Usage: gencmd GET [filenamestem] | commands\n"
	     "GET reads waveforms back from the scope to file.\n"
	     "commands is a set of valid commands separated by ;\n" );
    return( 1 );
  }

  if( argc == 3 )
    sprintf( outfilestem, "%s", argv[2] );
  else
    sprintf( outfilestem, "/tmp/rigol" );

  // Open communications with the oscilloscope.
  Rigol_DS1000* scope = new Rigol_DS1000( 0 );
  if( scope != NULL ){
    const char* errstr = scope->error_string();
    if( errstr != NULL ){
      fprintf( stderr, "Error: %s\n", errstr );
      return( 1 );
    }

    // If command is GET, acquire and read waveforms.
    if( strstr( argv[1], "GET" ) != NULL ){
      int chan1pts, chan2pts, maxpts;
      if( scope->acquire_channels( &chan1pts, &chan2pts, &maxpts ) ){
	float* vert, deltat, hoffset, voffset;
	printf( "Channel points: CH1:%d, CH2:%d, MAX:%d\n", chan1pts, chan2pts, maxpts );
	if( chan1pts > 0 ){
	  scope->get_channel( 1, &vert, &deltat, &hoffset, &voffset );
	  save_data( chan1pts, vert, deltat, outfilestem, 1 );
	}
	if( chan2pts > 0 ){
	  scope->get_channel( 2, &vert, &deltat, &hoffset, &voffset );
	  save_data( chan1pts, vert, deltat, outfilestem, 2 );
	}
      }
      else{
	errstr = scope->error_string();
	if( errstr != NULL )fprintf( stderr, "Error: %s\n", errstr );
	return( 1 );
      }
    }

    else{
      // Split the input commands up. Allow up to 20.
      char* cmds[20];
      char incmd[1024];
      memset( incmd, 0, 1024 );
      strncpy( incmd, argv[1], 1023 );
      int nparts = scope->split_string( incmd, cmds, 20 );
      for( int i = 0; i<nparts; i++ ){
	printf( "%d: %s\n", i, cmds[i] );

	// If command is a query, do a string query.
	if( strrchr( cmds[i], '?' ) != NULL ){
	  char* reply = scope->string_query( cmds[i] );
	  if( reply == NULL ){
	    errstr = scope->error_string();
	    if( errstr != NULL )fprintf( stderr, "Error: %s\n", errstr );
	    return( 1 );
	  }
	  else{
	    printf( "Reply=\"%s\"\n", reply );
	    fflush( stdout );
	  }
	}

	// Otherwise, send a command.
	else{
	  if( ! scope->command( cmds[i] ) ){
	    errstr = scope->error_string();
	    if( errstr != NULL )fprintf( stderr, "Error: %s\n", errstr );
	    return( 1 );
	  }
	}
      }
    }
  } // Over nparts cmds.

  return( 0 );
}
